@@ -957,6 +957,44 @@ static void test_hash_msdos_dosz_with_dosc()
   ASSERT_STR_EQUALS(hash_dosc, expected_dosc_md5);
 }
 
+static void test_hash_msdos_dosz_with_parent()
+{
+  size_t image_base_size, image_child_size;
+  uint8_t* image_base = generate_zip_file(&image_base_size);
+  uint8_t* image_child = generate_child_dosz_file(&image_child_size);
+  char hash_dosz[33], hash_dosc[33], hash_dosc2[33];
+  const char* expected_dosz_md5 = "623c759476b8b5adb46362f8f0b60769";
+  const char* expected_dosc_md5 = "ecd9d776cbaad63094829d7b8dbe5959";
+  const char* expected_dosc2_md5 = "cb55c123936ad84479032ea6444cb1a1";
+
+  /* Add base dosz file and child dosz file which will get hashed together */
+  mock_file(0, "base.dosz", image_base, image_base_size);
+  mock_file(1, "child.dosz", image_child, image_child_size);
+
+  /* test file hash */
+  int result_dosz = rc_hash_generate_from_file(hash_dosz, RC_CONSOLE_MS_DOS, "child.dosz");
+
+  /* test file hash with base.dosc also existing */
+  mock_file(2, "base.dosc", image_base, image_base_size);
+  int result_dosc = rc_hash_generate_from_file(hash_dosc, RC_CONSOLE_MS_DOS, "child.dosz");
+
+  /* test file hash with child.dosc also existing */
+  mock_file(3, "child.dosc", image_base, image_base_size);
+  int result_dosc2 = rc_hash_generate_from_file(hash_dosc2, RC_CONSOLE_MS_DOS, "child.dosz");
+
+  /* cleanup */
+  free(image_base);
+  free(image_child);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_dosz, 1);
+  ASSERT_NUM_EQUALS(result_dosc, 1);
+  ASSERT_NUM_EQUALS(result_dosc2, 1);
+  ASSERT_STR_EQUALS(hash_dosz, expected_dosz_md5);
+  ASSERT_STR_EQUALS(hash_dosc, expected_dosc_md5);
+  ASSERT_STR_EQUALS(hash_dosc2, expected_dosc2_md5);
+}
+
 /* ========================================================================= */
 
 static void test_hash_nes_32k()
@@ -1832,6 +1870,37 @@ static void test_hash_psp_video()
   ASSERT_NUM_EQUALS(result_iterator, 0);
 }
 
+static void test_hash_psp_homebrew()
+{
+  const size_t image_size = 3532124;
+  uint8_t* image = generate_generic_file(image_size);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_md5 = "fcde8760893b09e508e5f4fe642eb132";
+
+  mock_file(0, "eboot.pbp", image, image_size);
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_PSP, "eboot.pbp");
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "eboot.pbp", NULL, 0);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_md5);
+}
+
 static void test_hash_sega_cd()
 {
   /* the first 512 bytes of sector 0 are a volume header and ROM header. 
@@ -2165,19 +2234,31 @@ void test_hash(void) {
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/nes_game.zip", "9b7aad36b365712fc93728088de4c209");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/nes/game.zip", "9b7aad36b365712fc93728088de4c209");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "C:\\roms\\nes\\game.zip", "9b7aad36b365712fc93728088de4c209");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "C:\\roms\\NES\\game.zip", "9b7aad36b365712fc93728088de4c209");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "nes\\game.zip", "9b7aad36b365712fc93728088de4c209");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/snes/game.zip", "c8d46d341bea4fd5bff866a65ff8aea9");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/nes2/game.zip", "c8d46d341bea4fd5bff866a65ff8aea9");
 
+  /* we don't care that multiple aliases for the same system generate different hashes - the point is
+   * that they don't generate the same hash as an actual arcade ROM with the same filename. */
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/chf/game.zip", "6ef57f16562ea0c7f49d93853b313e32");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/channelf/game.zip", "7b6506637a0cc79bd1d24a43a34fa3b9");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/coleco/game.zip", "c546f63ae7de98add4b9f221a4749260");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/colecovision/game.zip", "47279207b94dbf2a45cb13efa56d685e");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/msx/game.zip", "59ab85f6b56324fd81b4e324b804c29f");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/msx1/game.zip", "33328d832dcb0854383cdd4a4565c459");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/pce/game.zip", "c414a783f3983bbe2e9e01d9d5320c7e");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/pcengine/game.zip", "49370c3cbe98bdcdce545c68379487db");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/sgx/game.zip", "db545ab29694bfda1010317d4bac83b8");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/supergrafx/game.zip", "5665c9ef4c2f6609d8e420c4d86ba692");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/tg16/game.zip", "8b6c5c2e54915be2cdba63973862e143");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/fds/game.zip", "c0c135a97e8c577cfdf9204823ff211f");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/gamegear/game.zip", "f6f471e952b8103032b723f57bdbe767");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/mastersystem/game.zip", "f4805afe0ff5647140a26bd0a1057373");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/sms/game.zip", "43f35f575dead94dd2f42f9caf69fe5a");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/megadriv/game.zip", "f99d0aaf12ba3eb6ced9878c76692c63");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/megadrive/game.zip", "73eb5d7034b382093b1d36414d9e84e4");
+  TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/genesis/game.zip", "b62f810c63e1cba7f5b7569643bec236");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/sg1000/game.zip", "e8f6c711c4371f09537b4f2a7a304d6c");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/spectrum/game.zip", "a5f62157b2617bd728c4b1bc885c29e9");
   TEST_PARAMS3(test_hash_filename, RC_CONSOLE_ARCADE, "/home/user/ngp/game.zip", "d4133b74c4e57274ca514e27a370dcb6");
@@ -2266,6 +2347,7 @@ void test_hash(void) {
   TEST(test_hash_msdos_dosz);
   TEST(test_hash_msdos_dosz_zip64);
   TEST(test_hash_msdos_dosz_with_dosc);
+  TEST(test_hash_msdos_dosz_with_parent);
 
   /* Neo Geo CD */
   TEST(test_hash_neogeocd);
@@ -2339,6 +2421,7 @@ void test_hash(void) {
   /* Playstation Portable */
   TEST(test_hash_psp);
   TEST(test_hash_psp_video);
+  TEST(test_hash_psp_homebrew);
 
   /* Pokemon Mini */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_POKEMON_MINI, "test.min", 524288, "68f0f13b598e0b66461bc578375c3888");
